#define MODULE "isr"

#include "assert.h"
#include "interrupts/Context.h"
#include "interrupts/IRQ.h"
#include "interrupts/Interrupts.h"
#include "io/Serial.h"
#include "log/Log.h"
#include "misc/hang.h"
#include "panic/Panic.h"
#include "std/stdio.h"
#include "thread/Scheduler.h"
#include "trace/StackTracer.h"

extern "C" void common_handler(Context* context)
{
    ASSERT(Interrupts::is_in_handler());
    if (context->number >= 0x20 && context->number < 0x30)
    {
        IRQ::interrupt_handler(context);
        return;
    }
    if (context->number == 13)
    {
        Interrupts::disable();

        if (context->cs == 0x8) { int_panic(context, "GPF in kernel task"); }
        else
        {
            kerrorln("General protection fault at RIP %lx, cs %ld, ss %ld, RSP %lx, error code %ld", context->rip,
                     context->cs, context->ss, context->rsp, context->error_code);
            kinfoln("Stack trace:");

            StackTracer tracer(context->rbp);
            tracer.trace_with_ip(context->rip);
            Scheduler::task_misbehave(context);
        }
    }
    if (context->number == 14)
    {
        Interrupts::disable();

        if (context->cs == 0x8) { int_panic(context, "Page fault in kernel task"); }
        else
        {
            kerrorln("Page fault in ring 3 (RIP %lx), while trying to access %lx, error code %ld", context->rip,
                     context->cr2, context->error_code);
            kinfoln("Stack trace:");

            StackTracer tracer(context->rbp);
            tracer.trace_with_ip(context->rip);

            Scheduler::task_misbehave(context);
        }
    }
    if (context->number == 8) { int_panic(context, "Double fault, halting"); }
    if (context->number == 48) { Scheduler::task_yield(context); }
    if (context->number == 49) { Scheduler::task_exit(context); }
    if (context->number == 50) { Serial::print((const char*)context->rdi); }
    if (context->number == 51) { Scheduler::task_sleep(context); }
    if (context->number == 256) { kwarnln("Unused interrupt"); }
    return;
}